/////////////////////////////////////////////////////////////////////////////
// Rev 30/12/2003                                               
// 
// 
//                            ENDEXE.CPP
//                            ----------
// 
// 
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
/////////////////////////////////////////////////////////////////////////////
// 
// 
// 
/////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <memory.h>
#include <stdio.h>
#include <string.h>

#include <stdlib.h>
#include <time.h>
#include <memory.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/locking.h>
#include <share.h>
#include <io.h>

#include "endexe.h"

#define EXE_SIZE_MARKER		"63861519-f125-4c95-a5f9-21f345fcd473"
#define EXE_SIZE_MARKER_LEN	sizeof(EXE_SIZE_MARKER)-1
#define EXE_SIZE_LEN		10

//////////////////////////////////////////////////////////////////////
static char * GetModifiedMarker()
{
    static int s_first = 1;
    static char s_szRealMarker[EXE_SIZE_MARKER_LEN];
    if( ! s_first ) return s_szRealMarker;
    s_first = 0;

    // On modifie le marker pour eviter qu'on tombe sur celui qui est
    // stocke qqpart ds l'exe
    memcpy( s_szRealMarker, EXE_SIZE_MARKER, EXE_SIZE_MARKER_LEN );
    for( int i = 0; i < EXE_SIZE_MARKER_LEN; i ++ )
    {
	s_szRealMarker[i] = s_szRealMarker[i] ^ 0x01;
    }
    return s_szRealMarker;
}

//////////////////////////////////////////////////////////////////////
static int GetExeSize( char * pszExeName )
{
    static int s_SizeExe;
    char * pExeContent;
    int nRealSize;
    int hFileExe;

    if( s_SizeExe ) return s_SizeExe;
    
    // On ouvre l'exe
    hFileExe = open( pszExeName, O_RDONLY | O_BINARY, 666 );    
    if( hFileExe == -1 )
    {
	return FALSE;
    }

    // On copie tout en memoire
    nRealSize = lseek( hFileExe, 0,  SEEK_END);
    pExeContent = (char *)malloc( nRealSize );
    if( ! pExeContent ) 
    {
	close( hFileExe );
	return FALSE;
    }
    lseek( hFileExe, 0,  SEEK_SET);
    read( hFileExe, pExeContent, nRealSize );
    close( hFileExe );

    // On recherche si la taille de l'exe existe
    for( int i = nRealSize - EXE_SIZE_MARKER_LEN - EXE_SIZE_LEN; i > 0; i -- )
    {
	if( memcmp( pExeContent + i, GetModifiedMarker(), EXE_SIZE_MARKER_LEN ) == 0 )
	{
	    char szSize[EXE_SIZE_LEN];
	    memcpy( szSize, pExeContent + i + EXE_SIZE_MARKER_LEN, EXE_SIZE_LEN );
	    s_SizeExe = atoi( szSize );
//MessageBox( NULL, "Found!", "Found!", 0 );
	    break;
	}
    }
    free( pExeContent );

    // Si on a pas trouve, il s'agit du premier appel
    if( s_SizeExe == 0 )
    {
	s_SizeExe = nRealSize + EXE_SIZE_MARKER_LEN + EXE_SIZE_LEN;
    }
    return s_SizeExe;
}

//////////////////////////////////////////////////////////////////////
static BOOL WriteExeSize( char * pszExeName )
{
    int hFileExe;
    int SizeExe;
    char szSize[EXE_SIZE_LEN];
    
    // On recupere la taille de l'exe
    SizeExe = GetExeSize( pszExeName );
    if( ! SizeExe ) return FALSE;

    // Ouverture
    hFileExe = open( pszExeName, O_RDWR | O_BINARY, 666 );
    if( ! hFileExe ) return FALSE;

    // On se deplace a la fin
    lseek( hFileExe, SizeExe - EXE_SIZE_MARKER_LEN - EXE_SIZE_LEN,  SEEK_SET);
    
    // On ecrit le marker
    write( hFileExe, GetModifiedMarker(), EXE_SIZE_MARKER_LEN );

    // On ecrit la taille
    sprintf( szSize, "%d", SizeExe );
    write( hFileExe, szSize, EXE_SIZE_LEN );

    close( hFileExe );
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
BOOL LoadExeHeader( char * pszExeName, BOOL *pfParamEnable, BOOL *pfExitOnClose,
		   BOOL *pfCloseWhenLastImage, int *pTimeSecClose  )
{
    int hFileExe; 
    int nRead;
    HeaderEndExe Header;
    int SizeExe;    
    char szExeName [MAX_PATH];
    
    // On recupere la taille de l'exe
    SizeExe = GetExeSize( pszExeName );
    if( ! SizeExe ) return FALSE;

    // Si on passe NULL, exe courant
    if( pszExeName == NULL )
	GetModuleFileName( GetModuleHandle(NULL), szExeName , MAX_PATH );
    else
	strcpy( szExeName, pszExeName );
    
    // Ouverture
    hFileExe = open( szExeName, O_RDONLY | O_BINARY, 666 );    
    if( hFileExe == -1 )
    {
	return FALSE;
    }
    
    // Deplacement a la fin de l'exe (-> sans les images)
    lseek( hFileExe, SizeExe,  SEEK_SET);
    
    // lecture de l'entete
    nRead = read( hFileExe, &Header, sizeof(HeaderEndExe) );
    if( nRead == 0 )
	return FALSE;
    if( nRead > 0 && (nRead != Header.cbSize || strcmp( Header.Marker, MARKER_ENDEXE_HEADER)) )
    {
	char szMsg[MAX_PATH];
	
	sprintf( szMsg, "Exe file '%s' is corrupt", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	close( hFileExe );
	return FALSE;
    }
    
    // Renvoie a l'appelant 
    *pfParamEnable = Header.fParamEnable;
    *pfExitOnClose = Header.fExitOnClose;
    *pfCloseWhenLastImage = Header.fCloseWhenLastImage;
    *pTimeSecClose = Header.TimeSecClose;
    return TRUE;
} 



//////////////////////////////////////////////////////////////////////
int AddFileInExe( char * pszExeName, char * pszImgFile, char * pszWavFile, 
		 BOOL fParamEnable, BOOL fExitOnClose, char * pszTitle, int TimeSecClose, 
		 BOOL fCloseWhenLastImage, BOOL fWallPaper )
{
    char szMsg[MAX_PATH];	
    int  hFileImg, hFileExe, hFileWav;
    char * pszBuffer;
    int nActualSize;
    int nRead; 
    HeaderEndExe Header;
    RecordEndExe Record;
    int SizeExe;
    
    // On recupere la taille de l'exe
    SizeExe = GetExeSize( pszExeName );
    if( ! SizeExe ) 
    {
	sprintf( szMsg, "Can't retrieve exe size (file '%s')", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return FALSE;
    }

    // On ecrit la taille de l'exe
    if( ! WriteExeSize( pszExeName ) )
    {
	sprintf( szMsg, "Can't write exe size (file '%s')", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return FALSE;
    }
    
    // Ouverture
    hFileExe = open( pszExeName, O_RDWR | O_BINARY, 666 );    
    
    // Ouverture ok ?
    if( hFileExe == -1 )
    {
	int hOriginFile;
	// le fichier n'existe pas, creons le
	char szOriginFile[MAX_PATH];
	
	// Ouverture de l'exe	
	GetModuleFileName( GetModuleHandle(NULL), szOriginFile, MAX_PATH );
	hOriginFile = open( szOriginFile, O_RDONLY | O_BINARY, 0666);    
	
	// test
	if( hOriginFile == -1 )
	{
	    sprintf( szMsg, "Can't open exe file '%s'", szOriginFile );
	    MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	    return FALSE;
	}
	
	// Alloc
	pszBuffer = (char *)malloc( SizeExe );
	if( pszBuffer == NULL )
	{
	    close( hFileExe );
	    MessageBox( NULL, "memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	    return FALSE;
	}
	
	// Creation nouveau fichier
	hFileExe = open( pszExeName, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);    
	int cbRead = min( SizeExe, lseek(hOriginFile, 0, SEEK_END) );
	lseek(hOriginFile, 0, SEEK_SET);
	read( hOriginFile, pszBuffer, cbRead );
	int cbWrite = write( hFileExe, pszBuffer, cbRead );
	
	// ferme l'origine
	close( hOriginFile );
	free( pszBuffer );
	close( hFileExe );
	
	// Ouverture
	hFileExe = open( pszExeName, O_RDWR | O_BINARY, 666 );    
	if( hFileExe == -1 )
	{
	    sprintf( szMsg, "Can't open exe file '%s'", pszExeName );
	    MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	    return FALSE;
	}
	
    }
    
    // Ouverture de l'image
    hFileImg = open( pszImgFile, O_RDONLY | O_BINARY, 0666);    
    if( hFileImg == -1 )
    {
	sprintf( szMsg, "Can't open the file '%s'", pszImgFile );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	close( hFileExe );
	return FALSE;
    }
    
    // Taille actuelle
    nActualSize = lseek( hFileExe, 0,  SEEK_END);
    
    // Si exe + petit, on comble par des 0
    if( nActualSize < SizeExe )
    {
	char *pZero;
	
	pZero = (char *)calloc( SizeExe - nActualSize, 1 );
	if( pZero == NULL )
	{
	    MessageBox( NULL, "memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	    close( hFileExe );
	    close( hFileImg );
	    return FALSE;
	}
	write( hFileExe, pZero, SizeExe - nActualSize );
    } 
    
    // Deplacement a la fin de l'exe (-> sans les images)
    lseek( hFileExe, SizeExe,  SEEK_SET);
    
    // lecture de l'entete
    nRead = read( hFileExe, &Header, sizeof(HeaderEndExe) );
    if( nRead > 0 && (nRead != Header.cbSize || strcmp( Header.Marker, MARKER_ENDEXE_HEADER)) )
    {
	sprintf( szMsg, "Exe file '%s' is corrupt", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	close( hFileExe );
	close( hFileImg );
	return FALSE;
    }
    
    
    // Se repositionne pour recreer l'entete
    lseek( hFileExe, SizeExe,  SEEK_SET);
    
    // On recree l'entete
    memset( &Header, 0, sizeof(HeaderEndExe) );
    Header.cbSize = sizeof(HeaderEndExe);
    strcpy( Header.Marker, MARKER_ENDEXE_HEADER );
    Header.fParamEnable = fParamEnable;
    Header.fExitOnClose = fExitOnClose;
    Header.TimeSecClose = TimeSecClose;
    Header.fCloseWhenLastImage = fCloseWhenLastImage;
    write( hFileExe, &Header, sizeof(HeaderEndExe) );
    
    
    // On recherche le dernier element
    do
    {
	// Lecture de l'entete de l'enregistrement
	nRead = read( hFileExe, &Record, sizeof(RecordEndExe) );
	if( nRead > 0 && (nRead != Record.cbSize || strcmp( Record.Marker, MARKER_ENDEXE_RECORD)) )
	{
	    sprintf( szMsg, "Exe file '%s' is corrupt", pszExeName );
	    MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	    close( hFileExe );
	    close( hFileImg );
	    return FALSE;
	}
	
	// Ce n'est pas le dernier
	if( nRead > 0 )
	{
	    lseek( hFileExe, Record.cbImgFile,  SEEK_CUR);
	    lseek( hFileExe, Record.cbWavFile,  SEEK_CUR);
	}
    } while( nRead > 0 );
    
    /*    // On verifie qu'on est bien a la fin du fichier
    if( nActualSize != lseek( hFileExe, 0,  SEEK_CUR) )
    {
      sprintf( szMsg, "Exe file '%s' is corrupt ", pszExeName );
      MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
      close( hFileExe );
      return FALSE;
      }
    */
    
    // On cree un nouvel enregistrement
    char * pszName;
    memset( &Record, 0, sizeof(RecordEndExe) );
    Record.cbSize = sizeof(RecordEndExe);    
    Record.fWallPaper = fWallPaper;
    
    // Taille de l'image
    Record.cbImgFile = lseek( hFileImg, 0,  SEEK_END);
    
    // nom de l'image
    pszName = pszImgFile + strlen(pszImgFile) - 1;
    while( *pszName > 0 && *pszName != '/' && *pszName != '\\' && *pszName != ':' && pszName > pszImgFile ) pszName --;
    pszName ++;
    memcpy( Record.szImgFile, pszName, min(sizeof(Record.szImgFile) - 1, strlen(pszName)) );
    
    
    // ouverture du fichier son et taille si y'a
    hFileWav = -1;
    if( pszWavFile && pszWavFile[0] )
	hFileWav = open( pszWavFile, O_RDONLY | O_BINARY, 0666);    
    if( hFileWav != -1 )
    {
	// Taille 
	Record.cbWavFile = lseek( hFileWav, 0,  SEEK_END);
	
	// Nom
	pszName = pszWavFile + strlen(pszWavFile) - 1;
	while( *pszName > 0 && *pszName != '/' && *pszName != '\\' && *pszName != ':' && pszName > pszWavFile ) pszName --;
	pszName ++;
	memcpy( Record.szWavFile, pszName, min( sizeof(Record.szWavFile) - 1, strlen(pszName) ) );
    }
    
    // pour verification de coherence pendant la lecture
    strcpy( Record.Marker, MARKER_ENDEXE_RECORD );
    
    // Titre de la fenetre
    if( pszTitle && pszTitle[0] )
	strcpy( Record.szTitle, pszTitle );
    
    
    
    
    // Mise de l'image dans un buffer
    pszBuffer = (char *)malloc(Record.cbImgFile);
    if( pszBuffer == NULL )
    {
	MessageBox( NULL, "Memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	close( hFileExe );
	close( hFileImg );
	close( hFileWav );
	return FALSE;
    }
    
    
    // On ecrit l'image
    lseek( hFileImg, 0,  SEEK_SET);
    read( hFileImg, pszBuffer, Record.cbImgFile );
    
    // type de l'image
    if( pszBuffer[0] == (char)0xff && pszBuffer[1] == (char)0xd8 && pszBuffer[2] == (char)0xff )
	Record.fImgType = TYPE_IMG_JPG;
    else
	Record.fImgType = TYPE_IMG_BMP;
    
    // Ecriture du record
    write( hFileExe, &Record, sizeof(RecordEndExe) );
    
    // Ecriture de l'image
    write( hFileExe, pszBuffer, Record.cbImgFile );
    
    
    free( pszBuffer );
    
    
    
    // On ecrit le fichier son
    if( Record.cbWavFile )
    {
	// Mise dans un buffer
	pszBuffer = (char *)malloc(Record.cbWavFile);
	if( pszBuffer == NULL )
	{
	    MessageBox( NULL, "Memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	    close( hFileExe );
	    close( hFileImg );
	    close( hFileWav );
	    return FALSE;
	}
	lseek( hFileWav, 0,  SEEK_SET);
	read( hFileWav, pszBuffer, Record.cbWavFile );
        write( hFileExe, pszBuffer, Record.cbWavFile );
	free( pszBuffer );
    }
    
    close(hFileExe);
    close(hFileImg);
    close(hFileWav);
    return TRUE;
}
    
    
//////////////////////////////////////////////////////////////////////
BOOL RemoveAllFilesFromExe( char * pszExeName )
{
    //int  hFileExe;
    HANDLE hFile;
    
    int SizeExe;
    
    // On recupere la taille de l'exe
    SizeExe = GetExeSize( pszExeName );
    if( ! SizeExe ) 
    {
	char szMsg[MAX_PATH];
	sprintf( szMsg, "Can't retrieve exe size (file '%s')", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return FALSE;
    }

    // Recuperation du nom de l'exe et ouverture
    //GetModuleFileName( GetModuleHandle(NULL), szExeName, MAX_PATH );
    /*hFileExe = _open( szExeName, _O_RDWR | _O_BINARY, 0666);    
    
      // Deplacement fin de l'exe
      _lseek( hFileExe, 0,  SizeExe);
      
	// truncage
	_write( hFileExe, NULL, 0 );
    _close( hFileExe )*/
    
    hFile = CreateFile(
	pszExeName,
	GENERIC_WRITE,
	FILE_SHARE_WRITE,
	NULL,
	OPEN_EXISTING,
	FILE_ATTRIBUTE_NORMAL,
	NULL );
    
    hFile = (HANDLE)GetModuleHandle(NULL);
    
    // Ouverture ok ?
    if( hFile == INVALID_HANDLE_VALUE )
    {
	char szMsg[MAX_PATH];
	
	sprintf( szMsg, "Can't open exe file '%s'", pszExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return FALSE;
    }
    
    SetFilePointer( hFile, SizeExe, 0, FILE_BEGIN );
    SetEndOfFile( hFile  );
    
    CloseHandle( hFile );
    
    return TRUE;
}


//////////////////////////////////////////////////////////////////////
int CreateJpgAndWavFileInTempDir( char * pszImgFile, BOOL * pfImgType, 
    char * pszWavFile, BOOL * pfExitOnClose, char * pszTitle, int * pTimeSecClose, 
    BOOL *pfCloseWhenLastImage, BOOL * pfWallPaper )
{
    char szMsg[MAX_PATH];
    int  hFileExe;
    char szExeName[MAX_PATH];
    char * pszBuffer;
    int nActualSize;
    int nRead, nPos; 
    HeaderEndExe Header;
    RecordEndExe Record;
    int SizeExe;
    static int s_OffSet;	// Position actuelle
    
    
    // inits
    pszImgFile[0] = 0;
    pszWavFile[0] = 0;
    *pfExitOnClose = FALSE;
    pszTitle[0] = 0;
    *pTimeSecClose = 0 ;
    *pfCloseWhenLastImage = 0;
    *pfWallPaper = FALSE;
    
    
    // Recuperation du nom de l'exe 
    GetModuleFileName( GetModuleHandle(NULL), szExeName, MAX_PATH );
    
    // On recupere la taille de l'exe
    SizeExe = GetExeSize( szExeName);
    if( ! SizeExe ) 
    {
	sprintf( szMsg, "Can't retrieve exe size (file '%s')", szExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return FALSE;
    }
    
    // On initialise l'offset le premier coup
    if( s_OffSet == 0 ) s_OffSet = SizeExe;

    // Ouverture de l'exe
    hFileExe = open( szExeName, O_RDONLY | O_BINARY, 0666);    

    // Ouverture ok ?
    if( hFileExe == -1 )
    {	
	sprintf( szMsg, "Can't open exe file '%s'", szExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	return 0;
    }
    
    
    // Taille actuelle
    nActualSize = lseek( hFileExe, 0,  SEEK_END);
    if( nActualSize <= SizeExe )
    {
	MessageBox( NULL, "No images !!!", "Error", MB_ICONSTOP | MB_TASKMODAL );
	close(hFileExe);
	return 2;
    }
    
    
    // Deplacement a l'offset
    nPos = lseek( hFileExe, s_OffSet,  SEEK_SET);
    if( nPos == nActualSize )
    {
	// Retour debut
	nPos = lseek( hFileExe, SizeExe,  SEEK_SET);
	s_OffSet = SizeExe;
    }
    
    // Lecture de l'entete
    if( SizeExe == s_OffSet )
    {
	static BOOL fFirst = TRUE;
	// pour l'appelant
	read( hFileExe, &Header, sizeof(HeaderEndExe) );
	*pfExitOnClose = Header.fExitOnClose;
	*pTimeSecClose = Header.TimeSecClose;
	if( fFirst )
	{
	    *pfCloseWhenLastImage = FALSE;
	    fFirst = FALSE;
	}
	else
	{
	    *pfCloseWhenLastImage = Header.fCloseWhenLastImage;
	}
	s_OffSet += sizeof(HeaderEndExe);
    }
    
    // Lecture de d'enregistrement
    nRead = read( hFileExe, &Record, sizeof(RecordEndExe) );
    if( nRead <= 0 || Record.cbImgFile == 0 )
    {
	sprintf( szMsg, "Exe file '%s' is corrupt ", szExeName );
	MessageBox( NULL, szMsg, "Error", MB_ICONSTOP | MB_TASKMODAL );
	close( hFileExe );
	return 0;
    }
    
    // pour l'appelant
    *pfImgType = Record.fImgType;
    *pfWallPaper = Record.fWallPaper;
    strcpy( pszTitle, Record.szTitle );
    
    
    // Mise de l'image dans un buffer
    pszBuffer = (char *)malloc(Record.cbImgFile);
    if( pszBuffer == NULL )
    {
	MessageBox( NULL, "Memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	close(hFileExe);
	return 0;
    }
    
    int hFileCreate;
    char szTemp[MAX_PATH];
    read( hFileExe, pszBuffer, Record.cbImgFile );
    
    // On cree un fichier bidon pour l'image
    GetTempPath( sizeof(szTemp), szTemp );
    sprintf( pszImgFile, "%s~temp.jpg", szTemp );
    //MessageBox( NULL, pszImgFile, "pszImgFile", MB_OK );
    
    // Ecriture et fermeture
    hFileCreate = open( pszImgFile, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);    
    write( hFileCreate, pszBuffer, Record.cbImgFile );
    free( pszBuffer );
    close( hFileCreate );
    
    // idem pour le son
    if( Record.cbWavFile )
    {
	// Mise de l'image dans un buffer
	pszBuffer = (char *)malloc(Record.cbWavFile);
	if( pszBuffer == NULL )
	{
	    MessageBox( NULL, "Memory error !", "Error", MB_ICONSTOP | MB_TASKMODAL );
	    close(hFileExe);
	    return 0;
	}
	read( hFileExe, pszBuffer, Record.cbWavFile );
	sprintf( pszWavFile, "%s~temp.wav", szTemp );
	
	// Ecriture et fermeture
	hFileCreate = open( pszWavFile, O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0666);    
	write( hFileCreate, pszBuffer, Record.cbWavFile );
	close( hFileCreate );
	free( pszBuffer );
    }
    
    
    
    // Mise a jour de l'offset pour l'appelant
    s_OffSet = lseek( hFileExe, 0,  SEEK_CUR);
    
    
    // On libere tout
    close(hFileExe);
    return 1;
}